@@ -9,9 +9,16 @@ use std::sync::{
 };
 use std::time::Duration;
 
-use common::{Observable, RawFunc, SubscriptionFunc};
-use handler::{Handler, HandlerThread};
-use publisher::Publisher;
+#[cfg(feature = "for_futures")]
+use std::future::Future;
+#[cfg(feature = "for_futures")]
+use std::pin::Pin;
+#[cfg(feature = "for_futures")]
+use std::task::{Context, Poll, Waker};
+
+use super::common::{Observable, RawFunc, SubscriptionFunc};
+use super::handler::{Handler, HandlerThread};
+use super::publisher::Publisher;
 
 /**
 `Will` `trait` defines the interface which could do actions in its `Handler`.
@@ -78,6 +85,9 @@ pub struct WillAsync<T> {
     publisher: Arc<Mutex<Publisher<T>>>,
     started_alive: Arc<Mutex<(AtomicBool, AtomicBool)>>,
     result: Arc<Mutex<Option<T>>>,
+
+    #[cfg(feature = "for_futures")]
+    waker: Arc<Mutex<Option<Waker>>>,
 }
 
 impl<T> WillAsync<T> {
@@ -94,11 +104,17 @@ impl<T> WillAsync<T> {
             started_alive: Arc::new(Mutex::new((AtomicBool::new(false), AtomicBool::new(false)))),
             publisher: Arc::new(Mutex::new(Publisher::default())),
             result: Arc::new(Mutex::new(None)),
+
+            #[cfg(feature = "for_futures")]
+            waker: Arc::new(Mutex::new(None)),
         }
     }
 }
 
-impl<T: Clone + Send + Sync + 'static> Will<T> for WillAsync<T> {
+impl<T> Will<T> for WillAsync<T>
+where
+    T: Clone + Send + Sync + 'static,
+{
     fn is_started(&mut self) -> bool {
         let _started_alive = self.started_alive.clone();
         let started_alive = _started_alive.lock().unwrap();
@@ -160,6 +176,13 @@ impl<T: Clone + Send + Sync + 'static> Will<T> for WillAsync<T> {
             }
             alive.store(false, Ordering::SeqCst);
         }
+
+        #[cfg(feature = "for_futures")]
+        {
+            if let Some(waker) = self.waker.lock().unwrap().take() {
+                waker.wake()
+            }
+        }
     }
 
     fn add_callback(&mut self, subscription: Arc<Mutex<SubscriptionFunc<T>>>) {
@@ -175,6 +198,27 @@ impl<T: Clone + Send + Sync + 'static> Will<T> for WillAsync<T> {
     }
 }
 
+#[cfg(feature = "for_futures")]
+impl<T> Future for WillAsync<T>
+where
+    T: Clone + Send + Sync + 'static,
+{
+    type Output = Option<T>;
+
+    fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
+        if self.is_started() && (!self.is_alive()) {
+            Poll::Ready(self.result())
+        } else {
+            if !self.is_started() {
+                self.start();
+            }
+
+            self.waker.lock().unwrap().replace(cx.waker().clone());
+            Poll::Pending
+        }
+    }
+}
+
 /**
 `CountDownLatch` implements a latch with a value(> 0),
 waiting for the value counted down until <= 0
@@ -190,12 +234,18 @@ It's inspired by `CountDownLatch` in `Java`
 #[derive(Debug, Clone)]
 pub struct CountDownLatch {
     pair: Arc<(Arc<Mutex<u64>>, Condvar)>,
+
+    #[cfg(feature = "for_futures")]
+    waker: Arc<Mutex<Option<Waker>>>,
 }
 
 impl CountDownLatch {
     pub fn new(count: u64) -> CountDownLatch {
         CountDownLatch {
             pair: Arc::new((Arc::new(Mutex::new(count)), Condvar::new())),
+
+            #[cfg(feature = "for_futures")]
+            waker: Arc::new(Mutex::new(None)),
         }
     }
 
@@ -206,6 +256,13 @@ impl CountDownLatch {
             *started -= 1;
         }
         cvar.notify_one();
+
+        #[cfg(feature = "for_futures")]
+        {
+            if let Some(waker) = self.waker.lock().unwrap().take() {
+                waker.wake()
+            }
+        }
     }
 
     pub fn wait(&self) {
@@ -234,6 +291,21 @@ impl CountDownLatch {
     }
 }
 
+#[cfg(feature = "for_futures")]
+impl Future for CountDownLatch {
+    type Output = ();
+
+    fn poll(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
+        let &(ref remaining, _) = &*self.pair.clone();
+        if *remaining.lock().unwrap() > 0 {
+            Poll::Ready(())
+        } else {
+            self.waker.lock().unwrap().replace(cx.waker().clone());
+            Poll::Pending
+        }
+    }
+}
+
 /**
 `Queue` `trait` defined the interface which perform basic `Queue` actions.
 
@@ -321,7 +393,10 @@ impl<T> BlockingQueue<T> {
     }
 }
 
-impl<T: 'static + Send> Queue<T> for BlockingQueue<T> {
+impl<T> Queue<T> for BlockingQueue<T>
+where
+    T: 'static + Send,
+{
     fn offer(&mut self, v: T) {
         {
             let alive = &self.alive.lock().unwrap();
@@ -388,11 +463,44 @@ impl<T: 'static + Send> Queue<T> for BlockingQueue<T> {
     }
 }
 
+#[cfg(feature = "for_futures")]
+#[futures_test::test]
+async fn test_sync_future() {
+    let wa = WillAsync::new(move || 1);
+
+    assert_eq!(Some(1), wa.await);
+
+    let mut _h = HandlerThread::new_with_mutex();
+    let mut pub1 = Publisher::new_with_handlers(Some(_h.clone()));
+
+    let latch = CountDownLatch::new(4);
+    let latch2 = latch.clone();
+
+    let _ = pub1.subscribe(Arc::new(Mutex::new(SubscriptionFunc::new(move |_| {
+        println!("{:?}", "SS");
+        latch2.countdown();
+    }))));
+
+    {
+        let h = &mut _h.lock().unwrap();
+
+        println!("hh2");
+        h.start();
+        println!("hh2 running");
+    }
+
+    pub1.publish(1);
+    pub1.publish(2);
+    pub1.publish(3);
+    pub1.publish(4);
+
+    let _ = latch.await;
+}
+
 #[test]
 fn test_will_sync_new() {
     use std::thread;
     use std::time;
-    use sync::CountDownLatch;
 
     let latch = CountDownLatch::new(1);
     let latch2 = latch.clone();
